/*
	Function Generator with STC MCU
	
	GitHub: https://github.com/CreativeLau/Function_Generator_STC
	
	Copyright (c) 2020 Creative Lau (CreativeLauLab@gmail.com)

	Permission is hereby granted, free of charge, to any person obtaining a copy
	of this software and associated documentation files (the "Software"), to deal
	in the Software without restriction, including without limitation the rights
	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
	copies of the Software, and to permit persons to whom the Software is
	furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in all
	copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
	SOFTWARE.
	
	Version: V0.3
	MCU Type: STC15W4K32S4 @24MHz
	YouTube Tutorial: https://youtu.be/c7iSE34SG90
    Instructable: https://www.instructables.com/id/DIY-Function-Generator-With-STC-MCU-Easily
    Any question please contact CreativeLauLab@gmail.com
    YouTube Channel: https://www.youtube.com/c/CreativeLau

	Specification:
	Output: Single Channel
	Square Waveform Frequency: 1Hz~2MHz
	Sine Waveform Frequency: 1Hz~10kHz
	Amplitude: VCC, about 5V
	Load ability: Not available
	Display: LCD1602
	Controller: EC11 Encoder

	Interface:
	Bottom left shows the type of waveform(Square/Sine) and output status(ON/OFF)
	F: Frequency
	D: Duty of Square Waveform
	CD: Clock Division Coefficient (For information only)
	P: PWM frequency for generating Sine Waveform (For information only)
	Pt: Number of points for generating Sine Waveform (For information only)

	Operations:
	Single Click Encoder: Switch Frequency and Duty in Square Waveform Interface
	Double Click Encoder: Start/Stop Signal Output
	Long Press Encoder: Switch between Square Waveform/Sine Waveform/Voltage Information
	Rotate Encoder: Adjust Parameters

	Note: 
	2020-05-06 Update
	1. Fix an error in main funciton, cause the EC11 Encoder rotate disorder.
	2. Fix an error of interface display during switch frequency.
	
	η
	ߣ
	汾V0.3
	ƬͺţSTC15W4K32S4 @24MHz
	κϵCreativeLauLab@gmail.com
	BվƵ̳̣https://www.bilibili.com/video/BV12k4y197Qu
	ģȫͬڴĹע

	
	ͨ
	1Hz~2MHz
	Ҳ1Hz-10kHz
	ԼVCC5V
	޴
	ʾLCD1602
	ƣEC11

	棺
	½ʾͼ꣨/Ҳ״̬On/OFF
	FƵ
	Dռձ
	CDʱӷƵϵFor information only
	PҲPWMƵʣFor information only
	PtҲĵFor information only

	
	£лƵʺռձ
	˫رղ
	л/Ҳ/ѹʾ
	תڲ

	2020-05-06 
	1. mainѭлתҵĴ
	   mainwhileѭеUpdate_Flag=0;ŵǰ㣬ںڱתʱδж϶Update_Flag޷ȷLCD
	2. лƵʱʾ
*/

#include <reg51.h>
#include <intrins.h>
#include "lcd1602.h"
#include "wave.h"
#include "settings.h"
#include "delay.h"
#include "config_stc.h"
//#include "uart.h"
//#include "stdio.h"

#ifndef uint8
#define uint8 unsigned char
#endif

#ifndef int8
#define int8 char
#endif

#ifndef uint16
#define uint16 unsigned int
#endif

#ifndef uint32
#define uint32 unsigned long int
#endif

#define TIMER_0 1 //ʱ0ж
#define INT_1 2	  //ת ⲿж
#define INT_0 0	  // ⲿж

uint8 Timer0_Count;
bit Update_Flag = 1;

void main(void)
{
	//LCD Pin
	P1M1 &= 0x00; //P1Ϊ׼˫
	P1M0 &= 0x00; //P1Ϊ׼˫
	P0M1 &= 0x00; //P0Ϊ׼˫
	P0M0 &= 0x00; //P0Ϊ׼˫

	//źPin
	PWM3 = 0;	   //PWM3 P4.5͵ƽ
	PWM4 = 0;	   //PWM4 P4.4͵ƽ
	P4M1 |= 0x30;  //P4.4(PWM4_2),4.5(PWM3_2)Ϊ
	P4M0 &= ~0x30; //P4.4(PWM4_2),4.5(PWM3_2)Ϊ

	/* תж
	   Interrupt for Encoder Rotation */
	IT1 = 0; //ⲿж1ʽغ½
	PX1 = 1; //ⲿж1ȼ
	EX1 = 1; //ⲿж1

	/* ж
	   Interrupt for Encoder Click */
	IT0 = 1; //ⲿж0ʽ½
	PX0 = 1; //ⲿж0ȼ
	EX0 = 1; //ⲿж0

	/* ʱ0ڸµѹϢʱ
	   Timer 0 for updating the information of VCC*/
	TMOD &= 0xF0;  //öʱ0ģʽ 16λԶأKeildebugĻע⣬8051ľ13λģʽ
	AUXR &= ~0x80; //ʱ0ʱ12Tģʽ
	TL0 = 0xC0;	   //öʱֵ 24MHz 20ms
	TH0 = 0x63;	   //öʱֵ 24MHz 20ms
	ET0 = 1;	   //T0ж

	/* ʱ1С50HzPWM
	   Timer 1 for generate the PWM when frequency less than 50Hz*/
	TMOD &= 0x0F;  //ģʽ,0: 16λԶװ
	AUXR &= ~0x40; //12T
	ET1 = 1;	   //ж

	EA = 1; //ж

	//UartInit();
	//UartInit_interrupt();
	PWM_Hz_Pre = PWM_Hz;
	Wave_Shape_Pre = Wave_Shape;
	Lcd_Init();
	while (1)
	{
		if (Update_Flag)
		{
			/*	Update_FlagҪ㣬Update_LCD棬ɼUpdate_LCDĹٴδתжϵĻ
				ִUpdate_LCDжλUpdate_FlagȴˣLCDûˢ£ʾĲһ¡
				һִַUpdate_LCDǰжϹصִٴж, ĺִʱȽϳתʱʹøܿ١
				жظкôģԱںִйжδ޸ĵ
				ִйв޸ģܻɼңʱƬλ	*/
			Update_Flag = 0;
			Wave_OFF();
			//EX1 = 0;
			Update_LCD();
			Set_Wave_Shape();
			//IE1=0;
			//EX1 = 1;
		}
	}
}

/* תӦ
   Encoder Rotate */
void Scan_EC11(void)
{
	/* ת
	   Rotate clockwise */
	if ((EC11_A != EC11_B))
	{

		Change_Val(1);
	}
	/* ת
	   Rotate anticlockwise*/
	else if ((EC11_A == EC11_B))
	{
		Change_Val(0);
	}
}

/* תж
   Interrupt for Encoder rotation */
void INT1_interrupt(void) interrupt INT_1
{
	Delay500us();
	Scan_EC11();
	Update_Flag = 1;
	//Delay50ms();
	IE1 = 0;
}

/* ж
   Interrupt for Encoder click */
void INT0_interrupt(void) interrupt INT_0
{
	Delay5ms();
	if (!EC11_KEY)
	{
		/* 
		   Long Press */
		if (Delay500ms_long_click())
		{
			Wave_Shape++;
			if (Wave_Shape > WAVE_NUM)
				Wave_Shape = 0;
			if (Wave_Shape == 2)
				Options = 1;
			WAVE_ON = 0;
			Clear_LCD_Flag = 1;
		}
		/* ˫
		   Double click */
		else if (Delay200ms_double_click())
		{
			if (Wave_Shape > 0)
			{
				WAVE_ON = ~WAVE_ON;
			}
		}
		/* 
		   Single click */
		else
		{
			if (Wave_Shape == 1)
				Options = ~Options;
		}
		Update_Flag = 1;
	}
	Delay5ms();
	IE0 = 0;
}

/* µѹϢʱж
   Timer interrupt for update voltage information */
void TIMER0_interrupt() interrupt TIMER_0
{
	if (++Timer0_Count > 200) //200x20=4000ms
	{
		Timer0_Count = 0;
		Update_Flag = 1;
	}
}
