/*
	wave.c
	
    Generate Square and Sine waveform functions

	GitHub: https://github.com/CreativeLau/Function_Generator_STC
	
	Copyright (c) 2020 Creative Lau (CreativeLauLab@gmail.com)

	Permission is hereby granted, free of charge, to any person obtaining a copy
	of this software and associated documentation files (the "Software"), to deal
	in the Software without restriction, including without limitation the rights
	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
	copies of the Software, and to permit persons to whom the Software is
	furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in all
	copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
	SOFTWARE.

    Note:
    2020-05-17 Update
    1. Fix an error in PWM_Config(), may casue wrong output PWM frequency when frequency less than 50Hz.
    1. PWM_Config()еһ㷨󣬻С50HzķƵʡ
*/

#include "wave.h"
#include "T_SineTable.h"

bit WAVE_ON = 0;                       //α־λ 1:ON 0:OFF
uint8 PWMCKS_PS = 0;                   //ϵͳʱӷƵϵ
uint32 PWM_Hz = 100;                   //PWMƵ
uint32 PWM_Hz_Pre;                     //¼һPWMƵ
uint32 PWM_Cycle;                      //PWM(ֵΪ32767)
int8 PWM_Duty = 50;                    //PWMռձ
uint32 PWM_Width;                      //PWMߵƽ
uint32 SIN_Hz = 100;                   //SINƵ
uint8 Wave_Shape = 1;                  //α־ 1: 2:Ҳ
uint8 Wave_Shape_Pre;                  //ϴβα־
uint16 PWM_Index = 0;                  //SPWM
uint16 T_SinTable_Current[SIN_POINTS]; //ԭʼSINֵµSIN
uint8 Sin_Table_Times = 1;             //SIN
uint32 PWM1_high, PWM1_low;
uint16 n, n_high, n_low;

void Set_PWMCKS_PS(void)
{
    if (PWM_Hz <= (FOSC / 0X7fff)) //0X7fff=3276715λPWMֵFOSC / 0X7fffʱӲƵPWM_HzСֵ
    {
        PWMCKS_PS = 0x0F;
    }
    else
    {
        PWMCKS_PS = 0x00;
    }
}

void Set_PWM_Cycle(void) //PWMڣ50HzʱʹǿPWMηֱɣSTC15W4KSTC8С50HzʹöʱGPIOת
{
    if (PWM_Hz < 50)
    {
        PWM_Cycle = FOSC / CKS_50HZ / PWM_Hz; //ʹöʱ1ѭɵƵ
    }
    else
    {
        PWM_Cycle = FOSC / (PWMCKS_PS + 1) / PWM_Hz; //ʹSTC15W4Kĸ߾PWMɸƵ
    }
}

void Set_PWM_Width(void) //PWMߵƽ
{
    PWM_Width = PWM_Cycle * PWM_Duty / 100;
}

void Set_Sin_Table_Times()
{
    Sin_Table_Times = 1;
    if (SIN_Hz > 6000)
        Sin_Table_Times = 100;
    else if (SIN_Hz > 5000)
        Sin_Table_Times = 50;
    else if (SIN_Hz > 3000)
        Sin_Table_Times = 40;
    else if (SIN_Hz > 2500)
        Sin_Table_Times = 25;
    else if (SIN_Hz > 1000)
        Sin_Table_Times = 20;
    else if (SIN_Hz > 500)
        Sin_Table_Times = 8;
    else if (SIN_Hz > 250)
        Sin_Table_Times = 4;
    else if (SIN_Hz > 100)
        Sin_Table_Times = 2;
}

void Wave_OFF(void) //رղ
{
    P_SW2 |= 0x80;  //xSFR
    PWMCR &= ~0x80; //رPWMģ 50HzķҲ
    PWMCR &= ~0x40; //ֹPWMж Ҳ
    PWMIF &= ~CBIF; //жϱ־
    P_SW2 &= ~0x80; //ָXRAM

    TR1 = 0; //رնʱ1С50Hzķ
    TF1 = 0; //ʱ1жϱ־

    //PWM IO״̬
    PWM3 = 0;      //PWM3 P4.5͵ƽ 
    PWM4 = 0;      //PWM4 P4.4͵ƽ	Ҳ
    P4M1 |= 0x30;  //P4.4(PWM4_2),4.5(PWM3_2)Ϊ
    P4M0 &= ~0x30; //P4.4(PWM4_2),4.5(PWM3_2)Ϊ
}

void PWM_Config(void)
{
    Set_PWMCKS_PS();
    Set_PWM_Cycle();
    Set_PWM_Width();
    if (WAVE_ON)
    {
        //PWM IO״̬
        PWM3 = 0;      //PWM3 P4.5͵ƽ
        P4M1 &= ~0x20; //P4.5Ϊ
        P4M0 |= 0x20;  //P4.5Ϊ
                       /* ռձΪ0ʱʼ͵ƽ
           Output low when duty cycle is 0*/
        if (PWM_Width == 0)
        {
            TR1 = 0;        //رնʱ1С50Hzķ
            TF1 = 0;        //ʱ1жϱ־
            PWMCR &= ~0x02; //PWMͨ3Ķ˿ΪGPIO
            PWM3 = 0;       //PWMͨ3ʼ͵ƽ
        }
        /* ռձΪ100%ʱʼߵƽ
           Output high when duty cycle is 100%*/
        else if (PWM_Width == PWM_Cycle)
        {
            TR1 = 0;        //رնʱ1С50Hzķ
            TF1 = 0;        //ʱ1жϱ־
            PWMCR &= ~0x02; //PWMͨ3Ķ˿ΪGPIO
            PWM3 = 1;       //PWMͨ3ʼߵƽ
        }
        /* PWMƵʴڵ50ʱʹǿPWM
           Use enhanced PWM waveform generator when PWM frequency higher than or equal to 50*/
        else if (PWM_Hz >= 50)
        {
            P_SW2 |= 0x80;       //xSFR
            PWMCKS = 0x00;       //PWMʱѡ
            PWMCKS |= PWMCKS_PS; //ϵͳʱӷƵΪPWMʱ
            PWMC = PWM_Cycle;    //PWM
            PWMCFG &= ~0x10;     //PWMʼƽ
            PWM3T1 = 0;          //һηת
            PWM3T2 = PWM_Width;  //ڶηת
            PWM3CR = 0x08;       //PWM3P4.5
            PWMCR = 0x02;        //ʹPWM3
            PWMCR &= ~0x40;      //ֹPWMж
            PWMCR |= 0x80;       //ʹPWMģ
            P_SW2 &= ~0x80;      //ָXRAM
        }
        /* PWMƵС50ʱʹöʱ
           Use timer when PWM frequency lower than 50*/
        else
        {
            PWMCR &= ~0x02;                                                      //PWMͨ3Ķ˿ΪGPIO
            PWM3 = 0;                                                            //PWMͨ3͵ƽ
            PWM1_high = PWM_Width;                                               //ߵƽʱ
            PWM1_low = PWM_Cycle - PWM_Width;                                    //͵ƽʱ
            n_high = PWM1_high / 65536;                                          //ߵƽʱĴ
            n_low = PWM1_low / 65536;                                            //͵ƽʱĴ
            PWM1_high = 65535 - PWM1_high % 65536 + FOSC * 2 / 10000 / CKS_50HZ; //ʱֵΪжPWMƽ仯ʱ200us
            PWM1_low = 65535 - PWM1_low % 65536 + FOSC * 2 / 10000 / CKS_50HZ;   //ʱֵΪжPWMƽ仯ʱ200us
            //㷨֮ǰдˣƵʴ
            if (PWM1_high > 65535) //Ķʱֵ65535ٴ
            {
                n_high--;
                PWM1_high -= 65535;
            }
            if (PWM1_low > 65535) //Ķʱֵ65535ٴ
            {
                n_low--;
                PWM1_low -= 65535;
            }
            n = n_low;
            TH1 = (uint8)(PWM1_low >> 8); //͵ƽװص͵ƽʱ䡣
            TL1 = (uint8)PWM1_low;
            TR1 = 1; //ʱ1ʼ
        }
    }
    else
    {
        Wave_OFF();
    }
}

void Sin_Wave_Config(void)
{
    int i;
    float Sin_Cycle_times; //SPWM_interruptжϺҪʱ䣬SPWMƵ120KSTC15W4K32S4ޣٸ߾ͨSPWM_interruptռձ
    Set_Sin_Table_Times();
    PWM_Hz = SIN_Hz * SIN_POINTS / Sin_Table_Times;

    //T_SinTableֵǸSIN_TABLE_PWM_HZõڣΪFOSC/SIN_TABLE_PWM_HZ24MHzʱ150kHzӦ160
    //ݵǰPWM_Hz¼ұʹҲ
    Sin_Cycle_times = SIN_TABLE_PWM_HZ * 1.0 / PWM_Hz; //ǰPWM_HzSIN_TABLE_PWM_HZı
    for (i = 0; i < SIN_POINTS; i += Sin_Table_Times)  //¼ұƫSIN_OFFSET
    {
        T_SinTable_Current[i] = T_SinTable[i] * Sin_Cycle_times + SIN_OFFSET;
    }
    Set_PWMCKS_PS();
    Set_PWM_Cycle();

    if (WAVE_ON)
    {
        //PWM IO״̬
        PWM4 = 0;      //PWM3 P4.5͵ƽ
        P4M1 &= ~0x10; //P4.4Ϊ
        P4M0 |= 0x10;  //P4.4Ϊ

        P_SW2 |= 0x80;       //xSFR
        PWMCR &= ~0x80;      //رPWMģ
        PWMCR &= ~0x40;      //ֹPWMж
        PWMIF &= ~CBIF;      //жϱ־
        PWMCKS = 0x00;       //PWMʱѡ
        PWMCKS |= PWMCKS_PS; //ϵͳʱӷƵΪPWMʱ
        PWMC = PWM_Cycle;    //PWM
        PWM_Index = 0;
        PWM4T1 = 0;                             //һηת
        PWM4T2 = T_SinTable_Current[PWM_Index]; //ڶηת
        PWM_Index += Sin_Table_Times;
        PWMCFG &= ~0x04; //PWM4ʼƽ
        PWM4CR = 0x08;   //PWM4P4.4ж
        PWMCR |= 0x04;   //ʹPWM4
        PWMCR |= 0x40;   //PWMж
        PWMCR |= 0x80;   //ʹPWMģ
        P_SW2 &= ~0x80;  //ָXRAM
    }
    else
    {
        Wave_OFF();
    }
}

/* 
   */
void Set_Wave_Shape(void)
{
    if (Wave_Shape_Pre != Wave_Shape)
    {
        Wave_Shape_Pre = Wave_Shape;
        if(Wave_Shape==0)
        {
            EX1 = 0; //رⲿж1ת
            TF0 = 0; //TF0־
            TR0 = 1; //ʱ0ʼʱ (ʱ0ΪVCC¼ʱ)
        }
        else if (Wave_Shape == 1)
        {
            PWM_Hz = PWM_Hz_Pre;
            EX1 = 1; //ⲿж1ת
            TR0 = 0; //رնʱ0 (ʱ0ΪVCC¼ʱ)
            TF0 = 0; //TF0־
        }
            
        else if (Wave_Shape == 2)
            PWM_Hz_Pre = PWM_Hz;
    }
    switch (Wave_Shape)
    {
    case 0:
        break;
    case 1:      //
        PWM_Config();
        break;
    case 2: //Ҳ
        Sin_Wave_Config();
        break;
    }
}

/* ҲSPWMж
   SPWM Interrupt for generating the sine waveform*/
void SPWM_interrupt(void) interrupt SPWM_VECTOR
{
    PWMIF &= ~CBIF; //жϱ־
    _push_(P_SW2);  //SW2
    P_SW2 |= 0x80;  //XFR
    PWM4T2 = T_SinTable_Current[PWM_Index];
    if ((PWM_Index += Sin_Table_Times) >= SIN_POINTS)
        PWM_Index = 0;
    _pop_(P_SW2); //ָSW2
}

/* 50HzPWMʹTimer1жϲ
   Generate PWM below 50Hz by Timer1 Interrupt*/
void TIMER1_interrupt(void) interrupt TIMER_1
{
    TR1 = 0;
    if (n-- == 0)
    {
        PWM3 = !PWM3;
        Delay200us(); //ʱȴPWM3ƽ仯200us 24MHzӦ4800ڣڼʱֵ۳
        if (PWM3)
        {
            n = n_high;
            TH1 = (uint8)(PWM1_high >> 8); //ߵƽװظߵƽʱ䡣
            TL1 = (uint8)PWM1_high;
        }
        else
        {
            n = n_low;
            TH1 = (uint8)(PWM1_low >> 8); //͵ƽװص͵ƽʱ䡣
            TL1 = (uint8)PWM1_low;
        }
    }
    else
    {
        TH1 = 0x00;
        TL1 = 0x00;
    }
    TF1 = 0;
    TR1 = 1;
}
